/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import pilrcedit.control.*;
import java.awt.Graphics;
import java.awt.Image;
import java.io.*;
import java.util.*;

/**
 * Class representing a standard Palm Alert.
 */
public class Alert extends Form
{
  public static String[] IMAGE_FILES;

  public static final int INFORMATION = 0;
  public static final int CONFIRMATION = 1;
  public static final int WARNING = 2;
  public static final int ERROR = 3;

  public static final int MESSAGE = 4;
  public static final int DEFAULTBUTTON = 5;
  public static final int BUTTONS = 6;


  public static String[] ALERT_TYPES;
  public static String PROP_MESSAGE;
  public static String PROP_ALERT_TYPE;
  public static String PROP_DEFAULTBUTTON;
  public static String PROP_BUTTONS;

  public static String DEFAULT_TITLE;
  public static String DEFAULT_TEXT;
  public static String DEFAULT_BUTTONS;


  int type = INFORMATION;
  String text;
  String[] textLines;
  String buttons;
  static Image[] images = new Image[4];

  public Alert(Project project)
  {
    ifdef=project.getIfDef();
    ifndef=project.getIfNDef();
    form = this;
    this.project = project;
    xExp.setValue(2);
    widthExp.setValue(156);
    title = DEFAULT_TITLE;
    setText(DEFAULT_TEXT);
    buttons = DEFAULT_BUTTONS;
    evaluateButtons();
    evaluatePosition();
  }

  public Alert(Project project, StreamTokenizer st) throws IOException
  {
    ifdef=project.getIfDef();
    ifndef=project.getIfNDef();
    form = this;
    this.project = project;
    readID(st);
    parseOptions(st);
    xExp.setValue(2);
    widthExp.setValue(156);
    evaluatePosition();
  }

  public void getProperties(Properties prop)
  {
    props = prop;
    prop.addProperty(PROP_TITLE,title);
    prop.addProperty(PROP_ID,id.getExpression());
    prop.addProperty(PROP_ALERT_TYPE,ALERT_TYPES,type);
    prop.addProperty(PROP_MESSAGE,text,prop.MULTILINE);
    prop.addProperty(PROP_BUTTONS,buttons,prop.LIST);
    prop.addProperty(PROP_IFDEF,ifdef);
    prop.addProperty(PROP_IFNDEF,ifndef);

  }

  public boolean setProperty(String name, String value, int index)
  {
    if (name==PROP_TITLE)
    {
      title = value;
      setDirty(-2, 0, width+4, 15);
      project.titleChanged(this);
    }
    else
    if (name==PROP_ALERT_TYPE)
    {
      type = index;
      setDirty(8, 19, 16, 16);
    }
    else
    if (name==PROP_MESSAGE)
      setText(value);
    else
    if (name==PROP_BUTTONS)
    {
      buttons = value;
      evaluateButtons();
    }
    else
      return super.setProperty(name, value, index);
    return true;
  }

  private void setText(String s)
  {
    getBounds();
    text = s;
    int lastLen=textLines==null?-1:textLines.length;
    textLines = TextTools.split(boldFont, text, 124);
    if (textLines.length!=lastLen)
    {
			heightExp.setValue(46 +
				Math.min(textLines.length, 10) * 11);
			yExp.setValue(158 - heightExp.getValue());
			evaluatePosition();
			if (firstObject != null)
			{
				firstObject.yExp.setValue(height - 18);
				firstObject.evaluatePosition();
			}
			setAllDirty();
		}
		else
		  setDirty(32,19,124,lastLen*11);
  }

  private boolean evaluateButtons()
  {
		String[] bs=TextTools.split(buttons,',');
    int cnt = 0;
    FormObject button = firstObject;
    if (bs!=null)
			for(int i=0;i<bs.length;i++)
			{
				if (button == null)
					add(button = new Button());
				button.label = bs[i];
				int width = button.autoWidth() - 3;
				if (width < 36)
					width = 36;
				button.widthExp.setValue(width);
				button.heightExp.setValue(13);
				if (cnt == 0)
				{
					button.xExp.setValue(5);
					button.yExp.setValue(height - 18);
				}
				else
				{
					button.xExp.setExpression("PREVRIGHT+6");
					button.yExp.setExpression("PREVTOP");
				}
				button.evaluatePosition();
				setDirty(button.getBounds());
				button = button.next();
				cnt++;
			}
    if (button != null)
    {
      setLast(button.prev());

			while (button != null)
			{
				setDirty(button.getBounds());
				button = button.next();
			}
		}
    return true;
  }

  protected boolean parseOption(StreamTokenizer st) throws IOException
  {
    frame = true;
    modal = true;
    int key=Keywords.get(this,st.sval);
    switch(key)
    {
      case HELPID:
        help=readNormalString(st);
        break;

      case DEFAULTBUTTON:
        st.nextToken();
        defaultButton = st.sval;
        break;

      case INFORMATION:
      case CONFIRMATION:
      case WARNING:
      case ERROR:
        type=key;
        break;

      case BEGIN:
        FormObject prev = null;
        st.nextToken();
        String end=Keywords.get(this,END);

        while (!(st.ttype == st.TT_WORD &&
          st.sval.equalsIgnoreCase(end)))
        {
          if (st.ttype != st.TT_WORD)
          {
            st.nextToken();
            continue;
          }
          key=Keywords.get(this,st.sval);
          switch(key)
          {
            case TITLE:
              title=readQuotedString(st);
              break;

            case MESSAGE:
              setText(readQuotedString(st));
              break;

            case BUTTONS:
              StringBuffer buttonsSB = null;
              String s = readQuotedString(st);
              while (s != null)
              {
                if (buttonsSB == null)
                  buttonsSB =
                    new StringBuffer(s);
                else
                  buttonsSB.append(",").append(s);
                s = readQuotedString(st);
              }
              buttons = (buttonsSB == null ? "":
                buttonsSB.toString());
              evaluateButtons();
              break;
          }
          st.nextToken();
        }
        break;

      default:
        return super.parseOption(st);
    }
    return true;
  }

  public void paint(Graphics g)
  {
    //if (images[type] == null)
    //  images[type]
    Image im= BitmapLoader.loadBWImage("images/"+
        IMAGE_FILES[type]);
    g.drawImage(im, x + 8, y + 19, null);
    for (int i = 0; i < 10 && i < textLines.length; i++)
    {
      boldFont.drawString(g, textLines[i], x + 32,
        y + 19 + i * 11, false);
    }
    super.paint(g);
  }

  public void write(PrintStream ps)
  {
    Keywords.setDefault(this);
    boolean endif=false;
    if (ifdef!=null&&!ifdef.equals(""))
    {
      ps.println(Keywords.get(IFDEF)+" "+ifdef);
      endif=true;
    }
    else
    if (ifndef!=null&&!ifndef.equals(""))
    {
      ps.println(Keywords.get(IFNDEF)+" "+ifndef);
      endif=true;
    }

    ps.print(Keywords.get(NAME));
    ps.println(id.toOutputString());
    if (help != null)
      ps.println(" "+Keywords.get(this,HELPID)+" "+help);
    if (defaultButton != null)
      ps.println(Keywords.get(this,DEFAULTBUTTON)+" "+defaultButton);

    ps.println(Keywords.get(this,type));

    ps.println(Keywords.get(this,BEGIN));
    if (title != null)
      ps.println("  "+Keywords.get(this,TITLE)+" "+TextTools.quote(title));
    ps.println("  "+Keywords.get(this,MESSAGE)+" \""+text + "\"");
    ps.print("  "+Keywords.get(this,BUTTONS)+" ");
    FormObject but = firstObject;
    while (but != null)
    {
      ps.print("\""+but.label + "\" ");
      but = but.next();
    }
    ps.println();
    ps.println(Keywords.get(this,END));
    if (endif)
      ps.println(Keywords.get(this,ENDIF));
    ps.println();
  }
}

