/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class BitmapFrame extends Frame
  implements ItemListener,FocusListener,ActionListener,TextListener,ChangeListener
{
  public static String TITLE;
  public static String NEW_BUTTON;
  public static String DELETE_BUTTON;

  Project project;
  FileDialog fd;
  Choice bitmaps;
  TextField id;
  TextField[] files=new TextField[4];
  Choice greycolor;
  TextField transparent;
  Choice compress;
  Checkbox colorTable;
  Object focus=null;
  Button[] browse=new Button[4];
  ImagePanel ip;
  ResourceCollection rc;
  Bitmap bitmap;
  int changecnt=0;

	public BitmapFrame(Project project)
	{
		super(TITLE);
    setLayout(new BorderLayout(0,10));
		boolean reg=Preferences.registerFrame(this);
		Panel p=new Panel(new BorderLayout(5,5));
    Panel p2=new Panel(new DynamicGridLayout(1,3,5,5));
		bitmaps=new Choice();
    bitmaps.addItemListener(this);
    p2.add(bitmaps);
    p2.add(new Label("ID",Label.RIGHT));
    p2.add(id=new TextField(10));
    id.addTextListener(this);
    id.addFocusListener(this);
    id.addActionListener(this);
    p.add("North",p2);
    DynamicGridLayout dgl=new DynamicGridLayout(4,3,5,5);
    dgl.setStyle(dgl.LABEL_FILL);
    p2=new Panel(dgl);
    p2.add(new Label("B&W",Label.RIGHT));
    p2.add(files[0]=new TextField());
    p2.add(browse[0]=new Button("Browse..."));
    p2.add(new Label("Grey",Label.RIGHT));
    p2.add(files[1]=new TextField());
    p2.add(browse[1]=new Button("Browse..."));
    //Panel p3=new Panel();
    greycolor=new Choice();
    greycolor.add("Grey16");
    greycolor.add("Color16");
    greycolor.addItemListener(this);
    p2.add(greycolor);
    //p2.add(new Label("Grey16",Label.RIGHT));
    p2.add(files[2]=new TextField());
    p2.add(browse[2]=new Button("Browse..."));
    p2.add(new Label("Color",Label.RIGHT));
    p2.add(files[3]=new TextField());
    p2.add(browse[3]=new Button("Browse..."));
    for(int i=0;i<4;i++)
    {
      files[i].addTextListener(this);
      files[i].addFocusListener(this);
      browse[i].addActionListener(this);
    }
    p.add(p2);

    p2=new Panel(dgl=new DynamicGridLayout(1,4,5,5));
    p2.add(new Label("Trans",Label.RIGHT));
    p2.add(transparent=new TextField(8));
    transparent.addTextListener(this);
    transparent.addFocusListener(this);
    compress=new Choice();
    compress.add("No Compress");
    compress.add("Compress");
    compress.add("Force Compress");
    compress.addItemListener(this);
    p2.add(compress);
    p2.add(colorTable=new Checkbox("Color Table"));
    colorTable.addItemListener(this);
    p.add("South",p2);

    add("North",p);
    add(ip=new ImagePanel());

    p=new Panel();
    Button b=null;
    p.add(b=new Button(NEW_BUTTON));
    b.addActionListener(this);
    p.add(b=new Button(DELETE_BUTTON));
    b.addActionListener(this);
    add("South",p);
    //open((Bitmap)rc.getResourceAt(0));
    project.addChangeListener(this);
    setProject(project);
		if (!reg)
		  pack();
	}

  public void objectChanged(Object source,int id,Object data)
  {
    if (source==project)
    {
      if (id==CLEARED||id==Project.LOADED_FILE)
        setProject(project);
    }
    else
    if (id==Bitmap.PARENT_DIR_CHANGED)
    {
      focus=null;
      for(int i=0;i<4;i++)
        files[i].setText(bitmap.files[i]);
    }
  }

  public Insets getInsets()
  {
    Insets insets=super.getInsets();
    return new Insets(insets.top+5,insets.left+5,insets.bottom+5,insets.right+5);
  }

  public void setProject(Project project)
  {
    this.project=project;
    bitmaps.removeAll();
    bitmaps.add("Icon");
    bitmaps.add("SmallIcon");
    rc=project.getBitmaps();
    for(int i=2,size=rc.size();i<size;i++)
      bitmaps.add(((Bitmap)rc.getResourceAt(i)).id.getExpression());
    bitmaps.select(0);
    setBitmap((Bitmap)rc.getResourceAt(0));
  }

  public void setBitmap(Bitmap b)
  {
    if (bitmap!=null)
      bitmap.removeChangeListener(this);
    bitmap=b;
    focus=null;
    if (bitmap==null)
    {
      int i=bitmaps.getSelectedIndex();
      bitmap=new Bitmap(Math.min(i,Bitmap.BITMAP));
      rc.set(i,bitmap);
    }
    bitmap.addChangeListener(this);

    if (bitmap.getType()==Bitmap.BITMAP)
    {
      if (!bitmaps.getSelectedItem().equals(b.id.getExpression()))
        bitmaps.select(b.id.getExpression());

      id.setText(bitmap.id.getExpression());
      id.setEnabled(true);
      greycolor.setEnabled(true);
      compress.setEnabled(true);
    }
    else
    {
      int index=bitmaps.getSelectedIndex();
      int type=bitmap.getType();
      if (type==Bitmap.ICON&&index!=0)
        bitmaps.select(0);
      else
      if (type==Bitmap.SMALLICON&&index!=1)
        bitmaps.select(1);
      id.setText("N/A");
      id.setEnabled(false);
      greycolor.setEnabled(false);
      compress.setEnabled(false);
    }

    for(int i=0;i<4;i++)
      files[i].setText(bitmap.files[i]);

    greycolor.select(bitmap.color16?1:0);

    if (bitmap.files[Bitmap.COLOR]!=null&&!bitmap.files[Bitmap.COLOR].equals(""))
    {
      transparent.setText(bitmap.getTransparentString());
      transparent.setEnabled(true);
    }
    else
    {
      transparent.setText("N/A");
      transparent.setEnabled(false);
    }
    compress.select(bitmap.getCompression());
    colorTable.setState(bitmap.useColorTable());

    ip.setImage(bitmap.getImage());
  }

  public void newBitmap()
  {
    System.out.println("New Bitmap");
    rc.add(new Bitmap(Bitmap.BITMAP));
    bitmaps.add("");
    bitmaps.select(rc.size()-1);
    setBitmap((Bitmap)rc.getResourceAt(rc.size()-1));
    id.selectAll();
    new Focuser(id).start();
  }

  public void actionPerformed(ActionEvent e)
  {
    String action=e.getActionCommand();
    if (action==NEW_BUTTON)
    {
      newBitmap();
    }
    else
    if (action==DELETE_BUTTON)
    {
      int index=bitmaps.getSelectedIndex();

      if (index>=2)
      {
        rc.remove(index);
        bitmaps.remove(index);
        if (index>=rc.size())
        {
          bitmaps.select(index=rc.size()-1);
        }
      }
      else
        rc.set(index,null);
      setBitmap((Bitmap)rc.getResourceAt(index));
    }
    else
    {
      Object o=e.getSource();
      if (bitmap==null)
        createBitmap();
      if (o==id)
      {
        try
        {
          bitmap.id.setExpression(id.getText());
        }
        catch(IllegalArgumentException ex)
        {
          project.addID(id.getText());
          bitmap.id.setExpression(id.getText());
          id.setForeground(Color.black);
          int index=bitmaps.getSelectedIndex();
          bitmaps.remove(index);
          bitmaps.insert(bitmap.id.getExpression(),index);
          bitmaps.select(index);
          rc.resourceIDChanged(bitmap);
        }
      }
      else
      {
        for(int i=0;i<4;i++)
        {
          if (o==browse[i])
          {
            if (fd==null)
              fd=new FileDialog(this);
            fd.setTitle("Choose Bitmap");
            fd.setMode(fd.LOAD);
            fd.setFile("*.bmp;*.pbm");
            fd.setDirectory(project.getParentDir());
            fd.show();
            if (fd.getFile()!=null)
            {
              focus=files[i];
              files[i].setText(project.convertToRelativeFilename(
                fd.getDirectory()+fd.getFile()));
            }
            break;
          }
        }
      }
    }
  }

  public void focusGained(FocusEvent e)
  {
    Object o=e.getSource();
    focus=o;
    if (bitmap==null)
      return;
    for(int i=0;i<4;i++)
      if (o==files[i])
      {
        ip.setImage(bitmap.getImage(i));
        return;
      }
  }

  public void focusLost(FocusEvent e)
  {
  }

  public void keyTyped(KeyEvent e) {}

  public void keyPressed(KeyEvent e) {}

  public void keyReleased(KeyEvent e) {}

  public void textValueChanged(TextEvent e)
  {
    Object o=e.getSource();
    if (bitmap==null)
      createBitmap();
    if (focus!=o)
      return;
    if (o==transparent)
    {
      if (bitmap.setTransparentString(transparent.getText()))
        ip.setImage(bitmap.getImage(Bitmap.COLOR));
      return;
    }
    else
    if (o==id)
    {
      try
      {
        bitmap.id.setExpression(id.getText());
        id.setForeground(Color.black);
        int index=bitmaps.getSelectedIndex();
        bitmaps.remove(index);
        bitmaps.insert(bitmap.id.getExpression(),index);
        bitmaps.select(index);
        rc.resourceIDChanged(bitmap);
      }
      catch(IllegalArgumentException ex)
      {
        id.setForeground(Color.red);
      }
    }
    for(int i=0;i<4;i++)
      if (o==files[i])
      {
        bitmap.setFile(i,files[i].getText());
        if (bitmap.isValid(files[i].getText()))
        {
          ip.setImage(bitmap.getImage(i));
          if (i==Bitmap.COLOR)
          {
            transparent.setText(bitmap.getTransparentString());
            transparent.setEnabled(true);
          }
        }
        else
        {
          ip.setImage(null);
          if (i==Bitmap.COLOR&&transparent.isEnabled())
          {
            transparent.setText("N/A");
            transparent.setEnabled(false);
          }
        }
      }
	}

	public void itemStateChanged(ItemEvent e)
	{
    Object o=e.getSource();
		if (o==bitmaps)
    {
      int index=bitmaps.getSelectedIndex();
      setBitmap((Bitmap)rc.getResourceAt(index));
    }
    else
    {
      if (bitmap==null)
        createBitmap();
      if (o==colorTable)
      {
        bitmap.setColorTable(colorTable.getState());
      }
      else
      if (o==compress)
      {
        bitmap.setCompression(compress.getSelectedIndex());
      }
      else
      if (o==greycolor)
      {
        bitmap.setColor16(greycolor.getSelectedIndex()==1);
        ip.setImage(bitmap.getImage(Bitmap.GREY16));
      }
    }
	}

  public void createBitmap()
  {
    int index=bitmaps.getSelectedIndex();
    rc.set(index,bitmap=new Bitmap(Math.min(index,Bitmap.BITMAP)));
    bitmap.addChangeListener(this);
  }
}

class ImagePanel extends Panel implements MouseListener
{
  boolean oneToOne=false;
  Dimension size=new Dimension(100,100);
  Image image;

  public ImagePanel()
  {
    addMouseListener(this);
  }

  public void setImage(Image i)
  {
    if (i!=image)
    {
      image=i;
      repaint();
    }
  }

  public Dimension getMinimumSize()
  {
    return size;
  }

  public Dimension getPreferredSize()
  {
    return size;
  }

  public void paint(Graphics g)
  {
    if (image==null)
      return;
    Dimension d=getSize();
    int w=d.width;
    int h=d.height;
    int iw=image.getWidth(null);
    int ih=image.getHeight(null);
    int dh,dw;
    if (oneToOne)
    {
      dw=iw;
      dh=ih;
    }
    else
    {
      double sr=w/(double)h;
      double ir=iw/(double)ih;

      if (sr>ir)
      {
        dh=h;
        dw=(int)(h*ir);
      }
      else
      {
        dw=w;
        dh=(int)(w/ir);
      }
    }
    g.drawImage(image,(w-dw)/2,(h-dh)/2,dw,dh,null);
  }

  /**
   * Invoked when the mouse has been clicked on a component.
   */
  public void mouseClicked(MouseEvent e) {}

  /**
   * Invoked when a mouse button has been pressed on a component.
   */
  public void mousePressed(MouseEvent e)
  {
    oneToOne=!oneToOne;
    repaint();
  }

  /**
   * Invoked when a mouse button has been released on a component.
   */
  public void mouseReleased(MouseEvent e) {}

  /**
   * Invoked when the mouse enters a component.
   */
  public void mouseEntered(MouseEvent e) {}

  /**
   * Invoked when the mouse exits a component.
   */
  public void mouseExited(MouseEvent e) {}

}
