/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import java.util.*;

/**
 * Class for holding ChangeListeners for an object and firing announcements
 */
public class ChangeListeners
{
  public Vector listeners;
  public Hashtable subCategories;

  /**
   * Adds a listener to this collection, listening for a specific subcategory.
   * This form is used when an object wants to listen for a specific change, for
   * example a preference string
   * @param subCategory the sub category to listen for.
   * @param listener the listener to be notified
   */
  public void add(Object subCategory,ChangeListener listener)
  {
    if (subCategories==null)
      subCategories=new Hashtable();
    Object o=subCategories.get(subCategory);
    if (o==null)
      subCategories.put(subCategory,listener);
    else
    if (o instanceof ChangeListeners)
      ((ChangeListeners)o).add(listener);
    else
    {
      ChangeListeners cl=new ChangeListeners();
      cl.add((ChangeListener)o);
      cl.add(listener);
      subCategories.put(subCategory,cl);
    }
  }

  public void remove(Object subCategory,ChangeListener listener)
  {
    if (subCategories==null)
      return;
    Object o=subCategories.get(subCategory);
    if (o==null)
      return;
    if (o instanceof ChangeListeners)
    {
      ((ChangeListeners)o).remove(listener);
    }
    else
    if (o==listener)
      subCategories.remove(subCategory);
  }

  /**
   * Adds a listener to be notified of changes.
   * @param listener the listener
   */
  public void add(ChangeListener listener)
  {
    if (listeners==null)
      listeners=new Vector();
    listeners.addElement(listener);
  }

  public void remove(ChangeListener listener)
  {
    if (listeners==null)
      return;
    listeners.removeElement(listener);
  }

  public void clear()
  {
    listeners=null;
  }

  public void fireObjectChanged(Object subCategory,Object changed,int changeType)
  {
    fireObjectChanged(subCategory,changed,changeType,null);
  }

  public void fireObjectChanged(Object subCategory,Object changed,int changeType,Object data)
  {
    if (subCategories==null)
      return;
    Object o=subCategories.get(subCategory);
    if (o==null)
      return;
    if (o instanceof ChangeListeners)
      ((ChangeListeners)o).fireObjectChanged(changed,changeType,data);
    else
      ((ChangeListener)o).objectChanged(changed,changeType,data);
  }

  public void fireObjectChanged(Object changed,int changeType)
  {
    fireObjectChanged(changed,changeType,null);
  }

  public void fireObjectChanged(Object changed,int changeType,Object data)
  {
    if (listeners==null)
      return;
    for(Enumeration e=listeners.elements();e.hasMoreElements();)
    {
      ChangeListener listener=(ChangeListener)e.nextElement();
      listener.objectChanged(changed,changeType,data);
    }
  }
}