/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import java.awt.*;
import java.awt.event.*;
import java.util.StringTokenizer;

public class GeneralDialog extends Dialog implements ActionListener
{
	private static String[] okCancelButs={"Ok","Cancel"};
	private static String[] okBut={"Ok"};

	int numButs;
	java.awt.Button[] buts=null;
	Panel buttonPanel;
	Panel labelPanel;
	Insets insets;
	String text;
	int pressed=-1;
  Frame owner;
	public GeneralDialog(Frame owner)
	{
		super(owner,true);
		this.owner=owner;
		((BorderLayout)getLayout()).setVgap(10);
		add(labelPanel=new Panel(new GridLayout(0,1)));
		Panel p=new Panel();
		p.add(buttonPanel=new Panel(new GridLayout(1,0,10,0)));
		add("South",p);
		setBackground(SystemColor.control);
	}

  public int showDialog()
  {
		show();
		return pressed;
	}

  public void show()
  {
		Rectangle r=owner.getBounds();
		Dimension d=getSize();
		setLocation(r.x+(r.width-d.width)/2,r.y+(r.height-d.height)/2);
		super.show();
	}

	public int getPressedButtonIndex()
	{
		return pressed;
	}

	public String getPressedButton()
	{
		if (buts==null||pressed==-1)
		  return null;
		return buts[pressed].getLabel();
	}

  public Insets getInsets()
  {
		if (insets==null)
		{
		  Insets i=(Insets)super.getInsets();
		  insets=new Insets(i.top+10,i.left+10,i.bottom+10,i.right+10);
		}
		return insets;
	}

	public void setText(String text)
	{
		this.text=text;
		sizeText();
	}

  public void sizeText()
  {
		if (text==null)
		  return;
		int width=buttonPanel.getPreferredSize().width;
		if (width<200)
		  width=200;
		Component[] labels=labelPanel.getComponents();
		FontMetrics fm=getFontMetrics(getFont());
		StringTokenizer st=new StringTokenizer(text," \n",true);
		String line=null;
		int pos=0;
		while(st.hasMoreTokens())
		{
			String s=st.nextToken();
			if (s.equals(" "))
			  continue;
			String newLine=(line==null?s:line+" "+s);
			boolean slashN=s.equals("\n");
			if (slashN||fm.stringWidth(newLine)>width)
			{
			  if (pos<labels.length)
			    ((java.awt.Label)labels[pos++]).setText(line);
			  else
			    labelPanel.add(new java.awt.Label(line==null?"":line));
			  if (slashN)
			    line="";
			  else
  			  line=s;
			}
			else
			  line=newLine;
		}
		if (pos<labels.length)
			((java.awt.Label)labels[pos++]).setText(line);
		else
    	labelPanel.add(new java.awt.Label(line==null?"":line));
		for(int i=labels.length-1;i>=pos;i--)
		  labelPanel.remove(i);
		pack();
	}

  public void setButtonOk()
  {
		setButtons(okBut);
	}

  public void setButtonsOkCancel()
  {
		setButtons(okCancelButs);
	}

	public void setButtons(String[] buttons)
	{
		if (buts==null)
		  buts=new java.awt.Button[buttons.length];
		else
		if (buts.length<buttons.length)
		{
			java.awt.Button[] temp=new java.awt.Button[buttons.length];
			System.arraycopy(buts,0,temp,0,numButs);
			buts=temp;
		}
		for(int i=0;i<buttons.length;i++)
		{
		  if (buts[i]==null)
		  {
		    buts[i]=new java.awt.Button(buttons[i]);
		    buts[i].addActionListener(this);
			}
		  else
		    buts[i].setLabel(buttons[i]);
		  if (i>=numButs)
		    buttonPanel.add(buts[i]);
		}
		for(int i=buttons.length;i<numButs;i++)
		  buttonPanel.remove(buts[i]);
		numButs=buttons.length;
		sizeText();
	}

	/**
	 * Invoked when an action occurs
	 */
	public void actionPerformed(ActionEvent e)
	{
		Object o=e.getSource();
		for(int i=0;i<buts.length;i++)
		  if (buts[i]==o)
		    pressed=i;
	  hide();
	}
}