/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import java.lang.reflect.*;
import java.io.*;
import java.util.*;

public class Keywords
{
  public static final int LOWER_CASE=0;
  public static final int MIXED_CASE=1;
  public static final int UPPER_CASE=2;
  public static final int PREFERENCES=3;

  private static int returnCase=MIXED_CASE;
  private static Class def;
  private static final String KEYWORDS_FILE="pilrc.key";
  private static final String BACKUP_LANGUAGE_FILE="english.key";
  private static Hashtable classes=new Hashtable();

  private Keywords()
  {
  }

  static
  {
    load(KEYWORDS_FILE,false);
    setCase(MIXED_CASE);
  }

  public static void load(String file)
  {
    load(file,false);
    if (!file.equals(BACKUP_LANGUAGE_FILE))
      load(BACKUP_LANGUAGE_FILE,true);
  }

  public static void load(String file,boolean onlyFillInGaps)
  {
    //System.out.println("Keywords.load("+file+","+onlyFillInGaps+")");
    try
    {
      InputStream is=Keywords.class.getResourceAsStream("keywords/"+file);
      if (is==null)
      {
        is=new FileInputStream(file);
        if (is==null)
          return;
      }
      StreamTokenizer st=new StreamTokenizer(new InputStreamReader(is));
      st.resetSyntax();
      st.wordChars(33,255);
      st.whitespaceChars(0, ' ');
      st.commentChar('#');
      st.quoteChar('"');
      st.quoteChar('\'');
      st.ordinaryChar('[');
      st.ordinaryChar(']');
      st.ordinaryChar(',');
      st.ordinaryChar('=');
      st.ordinaryChar('{');
      st.ordinaryChar('}');
      st.ordinaryChar('/');
      st.slashStarComments(true);
      st.eolIsSignificant(false);
      st.nextToken();
      while(st.ttype!=st.TT_EOF)
      {
        try
        {
          String constant=st.sval;
          st.nextToken();
          boolean bracket=st.ttype=='{';
          String cla,field=null;
          if (bracket)
          {
            cla=constant;
          }
          else
          {
            int dot=constant.lastIndexOf('.');
            cla=constant.substring(0,dot);
            field=constant.substring(dot+1);
          }
          Class c=Class.forName(cla);

          if (bracket)
          {
            int cnt=0;
            st.nextToken();
            while(st.ttype!='}'&&st.ttype!=st.TT_EOF)
            {
              getValues(st,c,st.sval,onlyFillInGaps);
              if (st.ttype==st.TT_EOL)
                st.nextToken();
            }
          }
          else
          {
            getValues(st,c,field,onlyFillInGaps);
          }
        }
        catch(Exception e)
        {
          System.out.println("Error parsing keywords file on line "+st.lineno());
          e.printStackTrace();
          System.exit(-1);
          st.eolIsSignificant(true);
          while(st.ttype!=st.TT_EOL&&st.ttype!=st.TT_EOF)
            st.nextToken();
          st.eolIsSignificant(false);
        }
        st.nextToken();
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  public static void setCase(int c)
  {
    if (c==PREFERENCES)
      c=Preferences.getIntPref("OutputCase",UPPER_CASE);
    returnCase=c;
  }

  private static void getValues(StreamTokenizer st,Class c,String field,boolean onlyFillInGaps) throws IOException
  {
    try
    {
      Field f=c.getField(field);

      int type=f.getType()==String.class?1:f.getType()==String[].class?2:0;
      st.nextToken();
      int conInt=type==0?f.getInt(null):-1;
      st.eolIsSignificant(true);
      boolean first=true;
      boolean ignore=onlyFillInGaps&&f.get(null)!=null;
      Vector vec=null;
      while (st.ttype!=st.TT_EOL&&st.ttype!=st.TT_EOF&&st.ttype!='}')
      {
        if (st.ttype==st.TT_WORD||st.ttype=='"'||st.ttype=='\'')
        {
          if (type==1)
          {
            if (!ignore&&first)
            {
              f.set(null,st.sval);
              first=false;
            }
          }
          else
          if (type==2)
          {
            if (!ignore)
            {
              if (vec==null)
                vec=new Vector();
              vec.addElement(st.sval);
            }
          }
          else
          {
            //System.out.println("  ="+st.sval);
            Object o=classes.get(c);
            ClassKeywords ck=(o instanceof ClassKeywords)?(ClassKeywords)o:null;
            if (ck==null)
            {
              ck=new ClassKeywords();
              classes.put(c,ck);
            }
            if (!onlyFillInGaps||ck.get(conInt)==null)
              ck.add(conInt,st.sval);
          }
        }
        st.nextToken();
      }
      if (vec!=null)
      {
        String[] array=new String[vec.size()];
        for(int i=0;i<array.length;i++)
          array[i]=(String)vec.elementAt(i);
        f.set(null,array);
      }

      if (type>0&&!ignore&&!classes.containsKey(c))
        classes.put(c,c); // hack to stop the class unloading
    }
    catch(IllegalAccessException e1)
    {
      System.out.println("Can't access field: "+st.sval+" (line "+st.lineno()+")");
      nextLine(st);
    }
    catch(NoSuchFieldException e2)
    {
      System.out.println("Can't find field: "+st.sval+" (line "+st.lineno()+")");
      nextLine(st);
    }
    st.eolIsSignificant(false);
    //if (st.ttype==st.TT_EOL)
    //  st.nextToken();
  }

  private static void nextLine(StreamTokenizer st) throws IOException
  {
    st.eolIsSignificant(true);
    while (st.ttype!='}'&&st.ttype!=st.TT_EOL&&st.ttype!=st.TT_EOF)
      st.nextToken();
    st.eolIsSignificant(false);
  }

  public static void setDefault(String c)
  {
    try
    {
      setDefault(Class.forName(c));
    }
    catch(ClassNotFoundException e)
    {
    }
  }

  public static void setDefault(Object o)
  {
    setDefault(o.getClass());
  }

  public static void setDefault(Class c)
  {
    def=c;
  }


  public static int get(String s)
  {
    return get(def,s);
  }

  public static String get(int i)
  {
    return get(def,i);
  }

  public static int get(Object o,String s)
  {
    return get(o.getClass(),s);
  }

  public static String get(Object o,int i)
  {
    return get(o.getClass(),i);
  }

  public static int get(Class c,String s)
  {
    //System.out.println("get("+c+","+s+")");
    if (c!=null&&c!=Object.class)
    {
      ClassKeywords ck=(ClassKeywords)classes.get(c);
      if (ck!=null)
      {
        int i=ck.get(s);
        if (i!=-1)
          return i;
      }
      return get(c.getSuperclass(),s);
    }
    //System.out.println("Can't find a match for "+s);
    return -1;
  }

  public static String get(Class c,int i)
  {
    if (c!=null)
    {
      ClassKeywords ck=(ClassKeywords)classes.get(c);
      if (ck!=null)
      {
        String s=ck.get(i);
        if (s!=null)
        {
          switch(returnCase)
          {
            case LOWER_CASE: return s.toLowerCase();
            case UPPER_CASE: return s.toUpperCase();
            case MIXED_CASE: return s;
            default : return s;
          }
        }
      }
      return get(c.getSuperclass(),i);
    }
    System.out.println("Can't find a match for "+i);
    Debug.debug(10);
    System.exit(-1);
    return null;
  }

}

class ClassKeywords
{
  Hashtable stringToInt=new Hashtable();
  Hashtable intToString=new Hashtable();

  public void add(int id,String value)
  {
    Integer i=new Integer(id);
    stringToInt.put(value.toLowerCase(),i);
    if (!intToString.containsKey(i))
      intToString.put(i,value);
  }

  public String get(int id)
  {
    return (String)intToString.get(new Integer(id));
  }

  public int get(String value)
  {
    Integer i=(Integer)stringToInt.get(value.toLowerCase());
    if (i!=null)
      return i.intValue();
    return -1;
  }
}