/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit;

import java.awt.*;
import java.awt.image.ImageProducer;
import java.awt.event.*;
import java.util.*;
import java.io.*;

public class PilrcEdit extends Frame implements ActionListener,ChangeListener
{
  public static final String PREF_RECENT_FILE_COUNT="Recent File Count";
  public static final String PREF_RECENT_FILE="Recent File ";
  public static final String PREF_LANGUAGE_FILE="Language File";

  public static String PILRCEDIT;
  public static String NEW_PROJECT;
  public static String READY;
  public static String SAVED;

  public static String CONFIRM;
  public static String SAVE_CHANGES;
  public static String MODIFIED_RELOAD;
  public static String[] OK_CANCEL;
  public static String[] YES_NO_CANCEL;

  public static String[] COLOR_DEPTHS;

  public static String FILE;
  public static String NEW;
  public static String OPEN;
  public static String SAVE;
  public static String SAVE_AS;
  public static String RELOAD;
  public static String PREFERENCES;
  public static String EXIT;
  public static String VIEW;
  public static String COLOR_DEPTH;
  public static String ZOOM;
  public static String FORM;
  public static String NEW_FORM;
  public static String ALERT;
  public static String NEW_ALERT;
  public static String OTHERS;
  public static String CONSTANTS;

	static int bgColor=new Color(123,140,90).getRGB(); //Color.green;//new Color(192,208,184);
  private static PilrcEdit instance;
	Project project;
	public Screen screen;
	Editor editor;
	java.awt.MenuBar mb;
  Menu fileMenu;
	Menu formMenu;
	Menu alertMenu;
	Hashtable forms;
	static java.awt.Label status;
	FileDialog fd;
	static boolean test;
	boolean showingForm=true;
	CheckboxMenuItemGroup formAlertGroup=null;
  CheckboxMenuItemGroup colorGroup;
	OptionsFrame optionsFrame;
	ConstantFrame constants;
  BitmapFrame bitmaps;
  Java2Interface j2;
  int numRecent;
  String[] recentFiles;

  public PilrcEdit(String file,String bitmap,int startForm)
  {
		super(PILRCEDIT+" - "+(file!=null?file:NEW_PROJECT));

    instance=this;

    setupJava2Stuff();

		boolean set=Preferences.registerFrame(PILRCEDIT,this);
		Preferences.setMainFrame(this);
	  screen=new Screen();
	  if (bitmap!=null)
      screen.setBitmap(BitmapLoader.loadImage(bitmap));
	  add(screen);

    if (j2!=null)
      j2.setupDropTarget(this,screen);

	  add("South",status=new java.awt.Label(READY));
		pack();

	  setVisible(true);
	  project=new Project();
    project.addChangeListener(this);
	  editor=new Editor(screen);

    ControlList controlList=new ControlList(editor);
    editor.addSelectionListener(controlList);

	  addKeyListener(editor);
	  screen.addKeyListener(editor);

    createMenus();

		if (file==null)
		  project.newProject();
		else
  	  project.open(file);
  	Form form=project.getFirstForm();
		formAlertGroup.select((CheckboxMenuItem)forms.get(form));
		screen.setForm(form);
	}

  public static PilrcEdit getPilrcEdit()
  {
    return instance;
  }

  /**
   * Check to see if we are running in a Java2 VM and
   * if so, create a Java2Interface which the application
   * can interact with.  Loading is done with Class.forName() so
   * jdk1.1 users can compile just the pilrcedit directory with no
   * errors.
   */
  private void setupJava2Stuff()
  {
    try
    {
      // look for a Java2 class
      Class.forName("java.awt.dnd.DropTarget");

      j2=(Java2Interface)Class.forName("pilrcedit.java2.Java2").newInstance();
    }
    catch(Exception e)
    {
      // don't worry, just means we aren't using java2
    }
  }

  public Editor getEditor()
  {
    return editor;
  }

  public static void setStatus(String s)
  {
		status.setText(s);
	}

  public void createMenus()
  {
		mb=new java.awt.MenuBar();
		MenuItem mi;
		CheckboxMenuItem cmi;
		Menu menu=fileMenu=new Menu(FILE);
		mi=new MenuItem(NEW,new MenuShortcut('N'));
		mi.addActionListener(this);
		menu.add(mi);
		mi=new MenuItem(OPEN,new MenuShortcut('O'));
		mi.addActionListener(this);
		menu.add(mi);
		mi=new MenuItem(SAVE,new MenuShortcut('S'));
		mi.addActionListener(this);
		menu.add(mi);
		mi=new MenuItem(SAVE_AS,new MenuShortcut('A'));
		mi.addActionListener(this);
		menu.add(mi);
		mi=new MenuItem(RELOAD,new MenuShortcut('R'));
		mi.addActionListener(this);
		menu.add(mi);
		mi=new MenuItem(PREFERENCES,new MenuShortcut('P'));
		mi.addActionListener(this);
		menu.add(mi);
    recentFiles=new String[Preferences.getIntPref(PREF_RECENT_FILE_COUNT,4)];
    numRecent=recentFiles.length;
    for(int i=0;i<recentFiles.length;i++)
    {
      String s=Preferences.getPref(PREF_RECENT_FILE+i);
      if (s==null)
      {
        numRecent=i;
        break;
      }
      else
      {
        if (i==0)
          menu.addSeparator();
        recentFiles[i]=s;
        mi=new MenuItem(s);
        mi.addActionListener(this);
        menu.add(mi);
      }
    }

    menu.addSeparator();
		mi=new MenuItem(EXIT,new MenuShortcut('X'));
		mi.addActionListener(this);
		menu.add(mi);
		mb.add(menu);

		menu=new Menu(VIEW);
		Preferences.getViewMenuWindows(menu);



    Menu colorMenu=new Menu(COLOR_DEPTH);
    CheckboxMenuItemGroup colorGroup=new CheckboxMenuItemGroup();
    colorGroup.addActionListener(this);
    int color=Preferences.getIntPref(Bitmap.PREF_COLOR_DEPTH,0);
    for(int i=0;i<4;i++)
    {
      cmi=new CheckboxMenuItem(COLOR_DEPTHS[i],color==i);
      colorGroup.add(cmi);
      colorMenu.add(cmi);
    }
    menu.add(colorMenu);
    menu.add(new MenuItem("-"));
		CheckboxMenuItemGroup zoomGroup=new CheckboxMenuItemGroup();
		zoomGroup.addActionListener(this);
		int zoom=screen.getZoom();
		for(int i=1;i<=4;i++)
		{
		  cmi=new CheckboxMenuItem(ZOOM+" "+i+"x",zoom==i);
			zoomGroup.add(cmi);
			menu.add(cmi);
		}
		mb.add(menu);

    formAlertGroup=new CheckboxMenuItemGroup();
    formAlertGroup.addActionListener(this);

		formMenu=new Menu(FORM);
		formMenu.add(mi=new MenuItem(NEW_FORM));
		mi.addActionListener(this);
		formMenu.add(new MenuItem("-"));
    mb.add(formMenu);

    alertMenu=new Menu(ALERT);
    alertMenu.add(mi=new MenuItem(NEW_ALERT));
    mi.addActionListener(this);
    alertMenu.add(new MenuItem("-"));
    mb.add(alertMenu);

		Menu othersMenu=new Menu(OTHERS);
		othersMenu.add(mi=new MenuItem(CONSTANTS));
		mi.addActionListener(this);
    othersMenu.add(mi=new MenuItem(BitmapFrame.TITLE));
    mi.addActionListener(this);
		mb.add(othersMenu);

		setMenuBar(mb);

		forms=new Hashtable();
	}

  /**
   * Updates the recent files list at the bottom of the file menu
   * @param s the file just opened.
   */
  private void updateRecentFiles(String s)
  {
    if (numRecent==0)
      fileMenu.insert("-",7);
    else
    if (s.equalsIgnoreCase(recentFiles[0]))
      return;

    int len=-1;
    for(int i=0;i<numRecent;i++)
      if (s.equalsIgnoreCase(recentFiles[i]))
      {
        len=i;
        fileMenu.remove(i+7);
        break;
      }
    if (len==-1)
    {
      len=Math.min(numRecent,recentFiles.length-1);
      if (numRecent<recentFiles.length)
        numRecent++;
      else
        fileMenu.remove(len+7);
    }
    System.arraycopy(recentFiles,0,recentFiles,1,len);
    recentFiles[0]=s;
    MenuItem mi=new MenuItem(s);
    mi.addActionListener(this);
    fileMenu.insert(mi,7);
  }

  /**
   * Checks to see if the current project has been modified and if so, prompts the
   * user to save changes.
   * @return false if Cancel was pressed, indicating the project should be left as it is,
   * or true to indicate that any necessary saving has been completed and the new operation
   * should be performed.
   */
  public boolean checkProjectSave()
  {
		if (project==null||!project.isDirty())
		  return true;
		GeneralDialog gd=new GeneralDialog(this);
		gd.setText(SAVE_CHANGES);
		gd.setButtons(YES_NO_CANCEL);
		gd.setTitle(CONFIRM);
		int ret=gd.showDialog();
		if (ret==0)
		{
		  saveProject(false);
		  return true;
		}
		else
		  return (ret==1);
	}

  private boolean saveProject(boolean saveAs)
  {
		if (fd==null)
			fd=new FileDialog(this);
		String dir=null;
	  String file=project.file;
	  if (file==null||saveAs)
	  {
			if (saveAs)
				fd.setTitle(SAVE_AS);
			else
				fd.setTitle(SAVE);
			fd.setMode(fd.SAVE);
			fd.setFile("*.rcp");
			fd.show();
			if (fd.getFile()==null)
			  return false;
			file=fd.getDirectory()+fd.getFile();
		}
		if (file!=null)
		{
			if (!file.toLowerCase().endsWith(".rcp"))
				file+=".rcp";
			setTitle(PILRCEDIT+" - "+file);
			project.save(file);
      if (saveAs)
        updateRecentFiles(file);
			setStatus(SAVED);
			return true;
		}
		return false;
	}

  public void openBitmapFrame(boolean newBitmap)
  {
    openBitmapFrame(null,newBitmap);
  }

  public void openBitmapFrame(Bitmap bitmap,boolean newBitmap)
  {
    if (bitmaps==null)
      bitmaps=new BitmapFrame(project);
    bitmaps.show();
    if (newBitmap)
      bitmaps.newBitmap();
    else
    if (bitmap!=null)
      bitmaps.setBitmap(bitmap);
  }

  public void open(File f)
  {
    if (!f.getName().endsWith(".rcp"))
      return;
    if (checkProjectSave())
    {
      project.open(f.getPath());
      setTitle(PILRCEDIT+" - "+f.getName());
      Form form=project.getFirstForm();
      if (form!=null)
        formAlertGroup.select((CheckboxMenuItem)forms.get(form));
      screen.setForm(form);
      updateRecentFiles(f.getPath());
    }
  }

  public void jumpToForm(Form form)
  {
    formAlertGroup.select((CheckboxMenuItem)forms.get(form));
    screen.setForm(form);
  }

  public void objectChanged(Object changed,int changeType,Object data)
  {
    if (changed==project)
    {
      switch(changeType)
      {
        case CLEARED:
          forms.clear();
          MenuItem newForm=formMenu.getItem(0);
          formMenu.removeAll();
          formMenu.add(newForm);
          newForm=alertMenu.getItem(0);
          alertMenu.removeAll();
          alertMenu.add(newForm);
          break;

        case Project.DIRTY:
          String s=getTitle();
          if (!s.endsWith(" *"))
            setTitle(s+" *");
          break;

        case Project.CLEAN:
          s=getTitle();
          if (s.endsWith(" *"))
            setTitle(s.substring(0,s.length()-2));
          break;

        case Project.FORM_ADDED:
        case Project.FORM_ADDED_JUMPTO:
          Form f=(Form)data;
          CheckboxMenuItem mi=new CheckboxMenuItem(f.toString());
          mi.setActionCommand(FORM+(project.getNumForms()-1));
          formAlertGroup.add(mi);
          if (f instanceof Alert)
          {
            if (alertMenu.getItemCount()==1)
              alertMenu.add(new MenuItem("-"));
            alertMenu.add(mi);
          }
          else
          {
            if (formMenu.getItemCount()==1)
              formMenu.add(new MenuItem("-"));
            formMenu.add(mi);
          }
          forms.put(f,mi);

          if (changeType==Project.FORM_ADDED_JUMPTO)
            jumpToForm(f);

          break;

        case Project.FORM_MODIFIED:
          f=(Form)data;
          mi=(CheckboxMenuItem)forms.get(f);
          if (mi!=null)
            mi.setLabel(f.toString());
          break;

        case Project.FORM_DELETED:
          f=(Form)data;
          mi=(CheckboxMenuItem)forms.get(f);
          Menu parent=(Menu)mi.getParent();
          parent.remove(mi);
          if (parent.getItemCount()==2)
            parent.remove(parent.getItem(1));
          Form form=project.getFirstForm();
          if (form!=null)
            formAlertGroup.select((CheckboxMenuItem)forms.get(form));
          screen.setForm(form);
          break;

        case Project.CONSTANTS_CHANGED:
          if (constants!=null)
            constants.refresh(project);
          break;

        case Project.LOADED_FILE:
          if (constants!=null)
            constants.refresh(project);
          if (bitmaps!=null)
            bitmaps.setProject(project);
      }
    }
  }

  public void quit()
  {
    if (checkProjectSave())
    {
      for(int i=0;i<numRecent;i++)
        Preferences.setPref(PREF_RECENT_FILE+i,recentFiles[i]);
      Preferences.save();
      System.exit(0);
    }
  }

	/**
	 * Invoked when an action occurs
	 */
	public void actionPerformed(ActionEvent e)
	{
		String action=e.getActionCommand();
    if (action==NEW)
    {
      if (checkProjectSave())
      {
        project.newProject();
        setTitle(PILRCEDIT+" - "+NEW_PROJECT);
        Form form=project.getFirstForm();
        formAlertGroup.select((CheckboxMenuItem)forms.get(form));
        screen.setForm(form);
      }
    }
    else
    if (action==OPEN)
    {
      if (checkProjectSave())
      {
        if (fd==null)
          fd=new FileDialog(this);
        fd.setTitle(OPEN);
        fd.setMode(fd.LOAD);
        fd.setFile("*.rcp");
        fd.show();
        if (fd.getFile()!=null)
        {
          project.open(fd.getDirectory()+fd.getFile());
          setTitle(PILRCEDIT+" - "+fd.getFile());
          Form form=project.getFirstForm();
          if (form!=null)
            formAlertGroup.select((CheckboxMenuItem)forms.get(form));
          screen.setForm(form);
          updateRecentFiles(fd.getDirectory()+fd.getFile());
        }
      }
    }
    else
    if (action==RELOAD)
    {
      boolean reload=true;
      if (project!=null&&project.isDirty())
      {
        GeneralDialog gd=new GeneralDialog(this);
        gd.setText(MODIFIED_RELOAD);
        gd.setButtons(OK_CANCEL);
        gd.setTitle(CONFIRM);
        if (gd.showDialog()!=0)
          reload=false;
      }
      if (reload)
      {
        Form f=project.reload(screen.getForm());
        formAlertGroup.select((CheckboxMenuItem)forms.get(f));
        screen.setForm(f);
      }
    }
    else
    if (action==SAVE)
    {
      saveProject(false);
    }
    else
    if (action==SAVE_AS)
    {
      saveProject(true);
    }
    else
    if (action==PREFERENCES)
    {
      if (optionsFrame==null)
        optionsFrame=new OptionsFrame();
      optionsFrame.show();
    }
    else
    if (action==EXIT)
    {
      quit();
    }
    else
    if (action==NEW_FORM)
    {
      Form form=project.newForm();
      formAlertGroup.select((CheckboxMenuItem)forms.get(form));
      screen.setForm(form);

    }
    else
    if (action==NEW_ALERT)
    {
      Alert alert=project.newAlert();
      formAlertGroup.select((CheckboxMenuItem)forms.get(alert));
      screen.setForm(alert);
    }
    else
    if (action==CONSTANTS)
    {
      if (constants==null)
        constants=new ConstantFrame(project);
      constants.show();
    }
    else
    if (action==BitmapFrame.TITLE)
    {
      openBitmapFrame(false);
    }
    else
    if (action.startsWith(ZOOM))
    {
      int zoom=action.charAt(ZOOM.length()+1)-'0';
      screen.setZoom(zoom);
      pack();
      if (zoom==1)
        pack();
    }
    else
    if (action.startsWith(FORM))
    {
      int currentForm=Integer.parseInt(action.substring(FORM.length()));
      screen.setForm(project.getFormAtIndex(currentForm));
    }
    else
    {
      for(int i=0;i<COLOR_DEPTHS.length;i++)
      {
        if (action==COLOR_DEPTHS[i])
        {
          Preferences.setPref(Bitmap.PREF_COLOR_DEPTH,i);
          Preferences.setPref(Bitmap.PREF_BGCOLOR,i==Bitmap.COLOR?Color.white.getRGB():bgColor);
          screen.refresh();
          return;
        }
      }
      open(new File(action));
		}
	}

  public static void main(String[] args)
  {
    if (!Preferences.jdk11)
      System.setProperty("sun.awt.exception.handler","pilrcedit.Debug");

    Keywords.load(Preferences.getPref(PREF_LANGUAGE_FILE,"english.key"));

		String file=null;
    String fileout=null;
		String bitmap=null;
		int startForm=0;
		for(int i=0;i<args.length;i++)
		{
			String lower=args[i].toLowerCase();
			if (lower.endsWith(".rcp"))
      {
        if (file==null)
  			  file=args[i];
        else
        if (fileout==null)
          fileout=args[i];
      }
			else
			if (lower.endsWith(".bmp"))
			  bitmap=args[i];
			else
			  try
			  {
					startForm=Integer.parseInt(args[i])-1;
				}
				catch(NumberFormatException e) {}
	  }
    if (fileout==null)
  		new PilrcEdit(file,bitmap,startForm);
    else
    {
      System.out.println(PILRCEDIT+" - opening \""+file+
        "\" and saving as \""+fileout+"\".");
      Project project=new Project();
      project.open(file);
      project.save(fileout);
      System.out.println("done.");
      System.exit(0);
    }
  }
}