/**
 * PilrcEdit - a graphical editor for PilRC files.
 * Copyright (C) 2000 Rob Nielsen
 * rnielsen@wn.com.au
 * http://www.wn.com.au/rnielsen/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 */
package pilrcedit.control;

import pilrcedit.*;
import java.awt.Graphics;
import java.awt.Color;
import java.io.*;
import java.util.Vector;

public class List extends FormObject
{
  public static String[] DEFAULT_LIST_ITEMS;
  public static String PROP_VISIBLE_ITEMS;
  public static String PROP_ITEMS;
  public static String PROP_POPPED_UP;
  public static String REMOVE_POPUP;
  public static String CREATE_POPUP;
  public static String POP_DOWN;
  public static final int VISIBLE_ITEMS=0;

	Expression visibleItems=new Expression(4);
	String[] listItems=DEFAULT_LIST_ITEMS;
  boolean dropdown=false;
  int slack=0;
  PopupTrigger trigger;
  boolean poppedup=false;

  protected Object clone() throws CloneNotSupportedException
  {
		List l=(List)super.clone();
		l.visibleItems=new Expression(visibleItems);
		return l;
	}

  public List()
  {
		setFont(0);
	}

  public boolean hasLabel()
  {
		return false;
	}

  public String getSelected()
  {
		return listItems[0];
	}

	public void parse(StreamTokenizer st) throws IOException
	{
		Vector listItemsVec=new Vector();
		String s=null;
    while((s=readQuotedString(st))!=null)
		{
		  listItemsVec.addElement(s);
		}
		int numItems=listItemsVec.size();
		listItems=new String[numItems];
		for(int i=0;i<listItems.length;i++)
			listItems[i]=(String)listItemsVec.elementAt(i);
    readID(st);
    readLocation(st);
    heightExp.setAuto();
    parseOptions(st);
    height=0;
    evaluatePosition();
	}

  public void setPopupTrigger(PopupTrigger trigger)
  {
		this.trigger=trigger;
		setVisible(trigger==null);
	}

  protected boolean parseOption(StreamTokenizer st) throws IOException
  {
		if (super.parseOption(st))
		  return true;
		if (Keywords.get(this,st.sval)==VISIBLE_ITEMS)
		{
			st.lowerCaseMode(false);
			st.nextToken();
			st.lowerCaseMode(true);
			visibleItems.setExpression(st.sval);
		}
		else
		  return false;
		return true;
  }

  public void getPopupMenuItems(Vector v)
  {
		super.getPopupMenuItems(v);
		v.addElement("-");
		if (trigger!=null)
		{
  		v.addElement(REMOVE_POPUP);
  		v.addElement(POP_DOWN);
		}
    else
			v.addElement(CREATE_POPUP);
	}

  public boolean isPoppedUp()
  {
		return poppedup&&trigger!=null;
	}

  public void popUp()
  {
		poppedup=true;
		setVisible(true);
		trigger.setVisible(false);
		requestFocus();
	}

  private void popDown()
  {
		poppedup=false;
		setVisible(false);
		trigger.setVisible(true);
		trigger.requestFocus();
	}

  public void doubleClicked()
  {
		if (trigger!=null&&poppedup)
		  popDown();
	}

	public boolean popupMenuItemSelected(String s)
	{
		if (s==POP_DOWN)
		  popDown();
		else
		if (s==REMOVE_POPUP)
		{
		  form.remove(trigger);
		  trigger=null;
		  form.setDirty(getBounds());
		}
		else
		if (s==CREATE_POPUP)
		{
			PopupTrigger pt=new PopupTrigger();
			form.insert(pt,prev());
			pt.setDimensions(xExp.getExpression(),yExp.getExpression(),
        Keywords.get(DimensionExpression.class,DimensionExpression.AUTO),
        Keywords.get(DimensionExpression.class,DimensionExpression.AUTO));
			pt.setList(this);
			pt.setLabel(getSelected());
      setDimensions(
        Keywords.get(PreviousExpression.class,PreviousExpression.PREVLEFT),
        Keywords.get(PreviousExpression.class,PreviousExpression.PREVTOP),null,null);
			popDown();
		}
		else
      return super.popupMenuItemSelected(s);
    return true;
	}

  public void getProperties(Properties prop)
  {
		super.getProperties(prop);
		prop.addProperty(PROP_VISIBLE_ITEMS,visibleItems.getExpression());
		prop.addProperty(PROP_ITEMS,TextTools.join(listItems,','),prop.LIST,true);
		if (trigger!=null)
  		prop.addProperty(PROP_POPPED_UP,boolStrings,0);
	}

  public void transform(int dx,int dy,int dw,int dh)
	{
		if (dh!=0)
		{
			slack+=dh;
			int fontHeight=getFont().getHeight();
			if (Math.abs(slack)>=fontHeight)
			{
				height+=slack;
				if (slack>=fontHeight)
				{
				  slack-=fontHeight;
				  visibleItems.setValue(visibleItems.getValue()+1);
				}
				else
				{
				  slack+=fontHeight;
				  visibleItems.setValue(visibleItems.getValue()-1);
				}
			  props.setProperty(PROP_VISIBLE_ITEMS,""+visibleItems.getValue());
			  evaluatePosition();
			}
			dh=0;
		}
		super.transform(dx,dy,dw,dh);
	}

  public boolean setProperty(String name,String value,int index)
  {
		if (name==PROP_VISIBLE_ITEMS)
		{
			visibleItems.setExpression(value);
			//heightExp.setValue("AUTO=autoHeight();
			//props.setProperty("Height",""+height);
		}
		else
		if (name==PROP_ITEMS)
		{
			listItems=TextTools.split(value,',');
		}
		else
		if (name==PROP_POPPED_UP)
			popDown();
		else
			return super.setProperty(name,value,index);
		evaluatePosition();
		return true;
	}

  public int autoWidth()
  {
		int width=0;
		for(int i=0;i<listItems.length;i++)
		{
			int w=getFont().stringWidth(listItems[i]);
		  if (w>width)
		    width=w;
		}
		return width+12;
	}

	public int autoHeight()
	{
		return getFont().getHeight()*visibleItems.getValue();
	}

  /**
   * Gets the cursor that should be displayed if the mouse
   * is positioned at the given point
   * @return a Cursor constant indicating an appropriate cursor.
   */
  public int getCursor(int px,int py)
  {
    if (visibleItems.getValue()==0)
      return java.awt.Cursor.MOVE_CURSOR;
    return super.getCursor(px,py);
	}

  public void paint(Graphics g)
  {
		int x1=x;
		int y1=y;
		int x2=x1+width-1;
		int y2=y1+height-1;
		g.setColor(usable||(trigger!=null)?Color.black:Color.lightGray);
		int visItems=visibleItems.getValue();

		if (visItems==0)
		{
		  g.drawLine(x,y,x+width,y);
		}
		else
		{
      Font font=getFont();
			g.fillRect(x,y,width,font.getHeight());

			for(int i=0;i<visItems&&i<listItems.length;i++)
			{
				font.drawString(g,listItems[i],x1+2,y1+font.getHeight()*i,i==0);
				if (i==visItems-1&&visItems<listItems.length)
				{
					Font.getFont(3).drawChar(g,(char)7,x2-9,y1+font.getHeight()*i+1);
				}
			}
			if (trigger!=null)
			{
				g.drawLine(x1-1,y1,x1-1,y2);
				g.drawLine(x1,y2+1,x2+1,y2+1);
				g.drawLine(x1+1,y2+2,x2,y2+2);
				g.drawLine(x1,y1-1,x2,y1-1);
				g.drawLine(x2+1,y1,x2+1,y2);
				g.drawLine(x2+2,y1+1,x2+2,y2);
			}
			else
				g.drawRect(x-1,y-1,width+1,height+1);
		}
		g.setColor(Color.black);
  }

	public void write(PrintStream ps)
	{
		StringBuffer sb=new StringBuffer();
		sb.append(getObjectType().toUpperCase());
		for(int i=0;i<listItems.length;i++)
		  sb.append(' ').append(TextTools.quote(listItems[i]));
    sb.append(id.toOutputString());
		sb.append(' ').append(Keywords.get(this,AT)).append(" (").append(xExp).append(" ").append(yExp);
		if (widthExp.isAuto()) // pilrc AUTO widths are broken
		  sb.append(" ").append(width);
		else
  		sb.append(" ").append(widthExp);
		sb.append(" ").append(heightExp);
		sb.append(")");
		writeOptions(sb);
		ps.println(sb.toString());
	}

	protected void writeOptions(StringBuffer sb)
	{
		super.writeOptions(sb);
		sb.append(' ').append(Keywords.get(this,VISIBLE_ITEMS)).append(' ').append(visibleItems);
	}
}